/*
 * program to test spline class
 *
 * $Author: claudio $
 *
 * $Revision: 1.1.1.1 $
 *
 * $Log: test_freespline.cpp,v $
 * Revision 1.1.1.1  2012-09-05 08:17:22  claudio
 * Interpolator utility classes
 *
 *
 */

#include <iostream>
#include <fstream>
#include "spline.h"
#include <cmath>


using namespace std;
using namespace Interpolator;

int main(){
	cout<< "test of Spline interpolator class" << endl;
	doubleVector x0,y0;
	const int NP=30;
	for(int i=0; i < NP; i++){
		x0.push_back(i*i*.02);
		y0.push_back(sin(i*i*.02));
	}
	Spline* test_spline = new Spline(x0,y0,NAN,0.0);
	{
		doubleVector a;
		vector<double> b;
		test_spline->get_base_points(a,b);
		cout<<"test get_base_points()"<<endl;
		for(int i=0; i < NP; i++ )
			if((x0[i] != a[i]) ||(y0[i] != b[i])){
				cout << i << " " << x0[i] << " " << a[i] << " | "<< y0[i] << " " << b[i] << endl;
				return -1;
			}
		
	}
	{
		cout << "test get_range()" << endl;
		double minx,maxx;
		test_spline->get_range(minx,maxx);
		if( (minx != x0[0]) || (maxx != x0[NP-1]) ){
			cout << minx << " " << x0[0] << " | " << maxx << " " << x0[NP-1] << endl;
			return -1;
		}
	}
	{
		cout << "test evaluate()" << endl;
		double xn=0.02;
		double yn=test_spline->evaluate(xn);
		cout << xn << " " << yn << endl;
		yn=test_spline->evaluate(x0[5]);
		cout << xn << " " << yn << " " << y0[5] << endl;	
	}
	{
		cout << "test evaluate()" << endl;
		double xn=0.02;
		double yn=test_spline->evaluate(xn);
		cout << xn << " " << yn << endl;
		yn=test_spline->evaluate(x0[5]);
		cout << xn << " " << yn << " " << y0[5] << endl;	
	}
	{	cout << "test evaluate()for plot" << endl;
		doubleVector X;
		doubleVector Y;
		double range=x0[NP-1];
		double nsteps=300;
		for(int i=0;i<=nsteps;i++){
			X.push_back((range*i)/nsteps);
			Y.push_back(test_spline->evaluate(X[i]));
		}
		//print out data
		cout<<"---------------------------"<<endl;
		for(int i=0; i < NP; i++) cout<<x0[i]<<"\t"<<y0[i]<<endl;
		cout<<"---------------------------"<<endl;
		for(int i=0; i < nsteps; i++) cout<<X[i]<<"\t"<<Y[i]<<endl;
		cout<<"+++++++++++++++++++++++++++"<<endl;
	}
	{	
		cout << "test evaluate()for doubleVector" << endl;
		doubleVector X;
		doubleVector Y;
		double range=x0[NP-1];
		double nsteps=300;
		for(int i=0;i<=nsteps;i++)
			X.push_back((range*i)/nsteps);
		Y=test_spline->evaluate(X);
		//print out data
		cout<<"###########################"<<endl;
		for(int i=0; i < nsteps; i++) cout<<X[i]<<"\t"<<Y[i]<<endl;
		cout<<"***************************"<<endl;
	}
	{
		cout << "test range exception" << endl;
		try{
			test_spline->evaluate(-10.0);
			cerr << "range_error not thrown!" << endl;
		}
		catch(std::range_error& ex){
			cout << "OK, catched expected exception: "<< ex.what() <<  endl;
		}
		catch(...){
			cerr << "unexpected exception thrown!" << endl;
		}
	}
	{
		cout << "test constructor with unequal lengths" << endl;
		try{
			doubleVector x,y;
			x.push_back(1.0);
			x.push_back(2.0);
			x.push_back(3.0);
			
			y.push_back(4.0);
			y.push_back(5.0);
			Spline spl(x,y,NAN,0.0);
			cerr << "length_error not thrown!" << endl;
		}
		catch(std::length_error& ex){
			cout << "OK, catched expected exception: "<< ex.what() <<  endl;
		}
		catch(...){
			cerr << "unexpected exception thrown!" << endl;
		}
	}
	{
		cout << "test constructor with out of order data" << endl;
		try{
			doubleVector x,y;
			x.push_back(1.0);
			x.push_back(2.0);
			x.push_back(1.5);
			
			y.push_back(4.0);
			y.push_back(5.0);
			y.push_back(6.0);
			Spline spl(x,y,NAN,0.0);
			cerr << "domain_error not thrown!" << endl;
		}
		catch(std::domain_error& ex){
			cout << "OK, catched expected exception: "<< ex.what() <<  endl;
		}
		catch(...){
			cerr << "unexpected exception thrown!" << endl;
		}
	}
	{
		cout << "Test copy constructor" << endl;
		Spline copy_spline(*test_spline);
		double minx,maxx,copyminx,copymaxx;
		test_spline->get_range(minx,maxx);
		copy_spline.get_range(copyminx,copymaxx);
		if( (minx != copyminx) || (maxx != copymaxx) ){
			cerr << "COPY FAILED!" << endl;
			return -1;
		}
		double range=x0[NP-1];
		double nsteps=100;
		for(int i=0;i<nsteps;i++){
			double val=(range*i)/nsteps;
			double y,ycopy;
			ycopy=copy_spline.evaluate(val);
			y=test_spline->evaluate(val);
			if(y != ycopy){
				cerr << "COPY FAILED " << val << " " << y << " " << ycopy << endl;
			}
		}
	}
	{
		cout << "Test copy assignment operator" << endl;
		Spline oper_spline=*test_spline;
		double minx,maxx,operminx,opermaxx;
		test_spline->get_range(minx,maxx);
		oper_spline.get_range(operminx,opermaxx);
		if( (minx != operminx) || (maxx != opermaxx) ){
			cerr << "ASSIGN COPY FAILED!" << endl;
			return -1;
		}
		double range=x0[NP-1];
		double nsteps=100;
		for(int i=0;i<nsteps;i++){
			double val=(range*i)/nsteps;
			double y,yoper;
			yoper=oper_spline.evaluate(val);
			y=test_spline->evaluate(val);
			if(y != yoper){
				cerr << "COPY FAILED " << val << " " << y << " " << yoper << endl;
			}
		}
	}
	cout << "test delete" << endl;
	delete test_spline;
	{
		cout << "test leaks - new/delete" << endl;
		Spline *dynspl;
		doubleVector x,y;
		x.push_back(1.0);
		x.push_back(2.0);
		x.push_back(3.5);
		
		y.push_back(4.0);
		y.push_back(5.0);
		y.push_back(6.0);
		for(int i=0; i < 100 ; i++){
			dynspl = new Spline(x,y,NAN,0.0);
			delete dynspl;
		}
	}
	{
		cout << "test leaks - stack" << endl;
		doubleVector x,y;
		x.push_back(1.0);
		x.push_back(2.0);
		x.push_back(3.5);
		
		y.push_back(4.0);
		y.push_back(5.0);
		y.push_back(6.0);
		for(int i=0; i < 100 ; i++){
			Spline stspl(x,y,NAN,0.0);
		}
	}
	
	{
		cout <<"test end conditions" << endl;
		std::ofstream outfile;
		outfile.open("splineval.csv",ios::out); //CHECK success
		if ( (outfile.rdstate() & ifstream::failbit ) != 0 ) return -1;
		doubleVector x,y;
		x.push_back(0.0);y.push_back(0.0);
		x.push_back(1.0);y.push_back(1.0);
		x.push_back(1.5);y.push_back(1.0);
		x.push_back(2.0);y.push_back(1.0);
		x.push_back(3.0);y.push_back(0.0);
		Spline nsp(x,y); //natural spline
		Spline zsp(x,y,0.0,0.0); // spline with 0 derivate start and stop
		Spline nzsp(x,y,NAN,0.0); // spline with 0 derivate at stop
		Spline znsp(x,y,0.0,NAN); // spline with 0 derivate at start
		Spline ddsp(x,y,-0.5,0.5); // spline with derivatives at start and stop
		double X;
		double Yn,Yz,Ynz,Yzn,Ydd;
		double range=3.0;
		int nsteps=300;
		for(int i=0;i<=nsteps;i++){
			X=((range*i)/nsteps);
			Yn=nsp.evaluate(X);
			Yz=zsp.evaluate(X);
			Ynz=nzsp.evaluate(X);
			Yzn=znsp.evaluate(X);
			Ydd=ddsp.evaluate(X);
			outfile<< X << "\t\t" << Yn << "\t\t" << Yz << "\t\t" << Ynz << "\t\t" << Yzn << "\t\t" << Ydd << std::endl;
		}
		outfile.close();
	}
	return 0;
}
