/*
 * program to test peridic spline class
 *
 * $Author: claudio $
 *
 * $Revision: 1.2 $
 *
 * $Log: test_periodicspline.cpp,v $
 * Revision 1.2  2017-03-06 13:13:20  claudio
 * use unsigned for comparison
 *
 * Revision 1.1.1.1  2012-09-05 08:17:22  claudio
 * Interpolator utility classes
 *
 *
 */

#include <iostream>
#include "periodicspline.h"
#include <cmath>

using namespace std;
using namespace Interpolator;

int main(){
	cout<< "test of periodicSpline interpolator class" << endl;
	doubleVector x0,y0;
	const int NP=15;
	const double k = (2*M_PI)/NP;
	for(int i=0; i <= NP; i++){
		x0.push_back(i*k);
		y0.push_back(sin(i*k));
	}
	periodicSpline* test_spline = new periodicSpline(x0,y0);
	{
		doubleVector a;
		doubleVector b;
		test_spline->get_base_points(a,b);
		cout<<"test get_base_points()"<<endl;
		for(unsigned int i=0; i < a.size()-1; i++ )
			if((x0[i] != a[i]) ||(y0[i] != b[i])){
				cout << i << " " << x0[i] << " " << a[i] << " | "<< y0[i] << " " << b[i] << endl;
				return -1;
			}
		
	}
	{
		cout << "test get_range()" << endl;
		double minx,maxx;
		test_spline->get_range(minx,maxx);
		int I=x0.size()-1;
		if( (minx != x0[0]) || (maxx != x0[I]) ){
			cout << minx << " " << x0[0] << " | " << maxx << " " << x0[I] << endl;
			return -1;
		}
	}
	{
		cout << "test evaluate()" << endl;
		double xn=0.02;
		double yn=test_spline->evaluate(xn);
		cout << xn << " " << yn << endl;
		yn=test_spline->evaluate(x0[5]);
		cout << xn << " " << yn << " " << y0[5] << endl;	
	}
	{
		cout << "test evaluate()" << endl;
		double xn=0.02;
		double yn=test_spline->evaluate(xn);
		cout << xn << " " << yn << endl;
		yn=test_spline->evaluate(x0[5]);
		cout << xn << " " << yn << " " << y0[5] << endl;	
	}
	{	cout << "test evaluate()for plot" << endl;
		doubleVector X;
		doubleVector Y;
		double minx;
		double range;
		test_spline->get_range(minx,range);
		double nsteps=100;
		for(int i=0;i<=nsteps;i++){
			X.push_back((range*i)/nsteps);
			Y.push_back(test_spline->evaluate(X[i]));
		}
		//print out data
		cout<<"---------------------------"<<endl;
		for(unsigned int i=0; i < x0.size(); i++) cout<<x0[i]<<"\t"<<y0[i]<<endl;
		cout<<"---------------------------"<<endl;
		for(int i=0; i <= nsteps; i++) cout<<X[i]<<"\t"<<Y[i]<<endl;
		cout<<"+++++++++++++++++++++++++++"<<endl;
	}
	{	
		cout << "test evaluate()for doubleVector" << endl;
		doubleVector X;
		doubleVector Y;
		double minx;
		double range;
		test_spline->get_range(minx,range);
		double nsteps=100;
		for(int i=0;i<=nsteps;i++)
			X.push_back((range*i)/nsteps);
		Y=test_spline->evaluate(X);
		//print out data
		cout<<"###########################"<<endl;
		for(int i=0; i <= nsteps; i++) cout<<X[i]<<"\t"<<Y[i]<<endl;
		cout<<"***************************"<<endl;
	}
	{
		cout << "test range exception" << endl;
		try{
			test_spline->evaluate(-10.0);
			cerr << "range_error not thrown!" << endl;
		}
		catch(std::range_error& ex){
			cout << "OK, catched expected exception: "<< ex.what() <<  endl;
		}
		catch(...){
			cerr << "unexpected exception thrown!" << endl;
		}
	}
	{
		cout << "test constructor with unequal lengths" << endl;
		try{
			doubleVector x,y;
			x.push_back(1.0);
			x.push_back(2.0);
			x.push_back(3.0);
			
			y.push_back(4.0);
			y.push_back(5.0);
			periodicSpline spl(x,y);
			cerr << "length_error not thrown!" << endl;
		}
		catch(std::length_error& ex){
			cout << "OK, catched expected exception: "<< ex.what() <<  endl;
		}
		catch(...){
			cerr << "unexpected exception thrown!" << endl;
		}
	}
	{
		cout << "test constructor with out of order data" << endl;
		try{
			doubleVector x,y;
			x.push_back(1.0);
			x.push_back(2.0);
			x.push_back(1.5);
			
			y.push_back(4.0);
			y.push_back(5.0);
			y.push_back(6.0);
			periodicSpline spl(x,y);
			cerr << "domain_error not thrown!" << endl;
		}
		catch(std::domain_error& ex){
			cout << "OK, catched expected exception: "<< ex.what() <<  endl;
		}
		catch(...){
			cerr << "unexpected exception thrown!" << endl;
		}
	}
	{
		cout << "Test copy constructor" << endl;
		periodicSpline copy_spline(*test_spline);
		double minx,maxx,copyminx,copymaxx;
		test_spline->get_range(minx,maxx);
		copy_spline.get_range(copyminx,copymaxx);
		if( (minx != copyminx) || (maxx != copymaxx) ){
			cerr << "COPY FAILED!" << endl;
			return -1;
		}
		double range;
		test_spline->get_range(minx,range);
		double nsteps=100;
		for(int i=0;i<nsteps;i++){
			double val=(range*i)/nsteps;
			double y,ycopy;
			ycopy=copy_spline.evaluate(val);
			y=test_spline->evaluate(val);
			if(y != ycopy){
				cerr << "COPY FAILED " << val << " " << y << " " << ycopy << endl;
			}
		}
	}
	{
		cout << "Test copy operator" << endl;
		periodicSpline copy_spline(*test_spline);
		copy_spline=*test_spline;
		double minx,maxx,copyminx,copymaxx;
		test_spline->get_range(minx,maxx);
		copy_spline.get_range(copyminx,copymaxx);
		if( (minx != copyminx) || (maxx != copymaxx) ){
			cerr << "COPY FAILED!" << endl;
			return -1;
		}
		double range;
		test_spline->get_range(minx,range);
		double nsteps=100;
		for(int i=0;i<nsteps;i++){
			double val=(range*i)/nsteps;
			double y,ycopy;
			ycopy=copy_spline.evaluate(val);
			y=test_spline->evaluate(val);
			if(y != ycopy){
				cerr << "COPY FAILED " << val << " " << y << " " << ycopy << endl;
			}
		}
	}
	cout << "test delete" << endl;
	delete test_spline;
	{
		cout << "test leaks - new/delete" << endl;
		periodicSpline *dynspl;
		doubleVector x,y;
		x.push_back(1.0);
		x.push_back(2.0);
		x.push_back(3.5);
		
		y.push_back(4.0);
		y.push_back(5.0);
		y.push_back(6.0);
		for(int i=0; i < 100 ; i++){
			dynspl = new periodicSpline(x,y);
			delete dynspl;
		}
	}
	{
		cout << "test leaks - stack" << endl;
		doubleVector x,y;
		x.push_back(1.0);
		x.push_back(2.0);
		x.push_back(3.5);
		
		y.push_back(4.0);
		y.push_back(5.0);
		y.push_back(6.0);
		for(int i=0; i < 100 ; i++){
			periodicSpline stspl(x,y);
		}
	}
	
	return 0;
}
